-- $Id: psizes.adb,v 1.1 2003/11/18 01:30:12 baker Exp baker $

with Ada.Text_IO; use Ada.Text_IO;
with Ada.Float_Text_IO; use Ada.Float_Text_IO;
with Ada.Numerics.Elementary_Functions;
with Sizes; use Sizes;
with Cycles; use Cycles;
with System_Type; use System_Type;
procedure Psizes is

  -- compute maximum aperiodic server sizes,
  -- for various scheduling models (T.P. Baker, August 1993)

  M: Integer;

  procedure Find_Server_Sizes(R: Runs) is

    generic
    with function Test(CS: Integer) return Boolean;
    function Max return Integer;

    function Max return Integer is
      L: Integer:= 0;
      U: Integer:= Server_Period;
      M: Integer;
    begin
      if Test(U) then return U;
      elsif not Test(L) then return L;
      else
        loop
          M:= (L+U)/2;
          if M=L then return L; end if;
          if Test(M) then L:=M; else U:=M; end if;
        end loop;
      end if;
    end;

    function Test_PLS(CS: Integer) return Boolean is
      U: Float:= 0.0;
    begin
      for I in Process_List'range loop
         U:= U+float(Processes(R)(I).Exec_Time)/float(Processes(R)(I).Period);
      end loop;
      return U+float(CS)/float(Server_Period)<=1.0;
    end;

    function Max_PLS is new Max(Test_PLS);

    function Test_DS(CS: Integer) return Boolean is
      Window: Integer:= 0;
      Work: Integer;
    begin
      for K in Process_List'range loop
        Window:= 0;
        Inner: -- check schedulability of Kth process
        loop
          Work:= (1+Ceiling_Div(Window-CS,Server_Period))*CS;
          for I in Process_List'first..K loop
            Work:= Work+Ceiling_Div(Window,Processes(R)(I).Period)
              *Processes(R)(I).Exec_Time;
          end loop;
          if Work<=Window then exit Inner;
          elsif Work>Processes(R)(K).Period then return False;
          else Window:= Work;
          end if;
        end loop Inner;
      end loop;
      return True;
    end;

    function Max_DS is new Max(Test_DS);

    function Test_RM(CS: Integer) return Boolean is
      Window: Integer;
      Work: Integer;
    begin
      for K in Process_List'range loop
        Window:= 0;
        Inner: -- check schedulability of Kth process
        loop
          Work:= Ceiling_Div(Window,Server_Period)*CS;
          for I in Process_List'first..K loop
            Work:= Work+Ceiling_Div(Window,Processes(R)(I).Period)
              *Processes(R)(I).Exec_Time;
          end loop;
          if Work<=Window then exit Inner;
          elsif Work>Processes(R)(K).Period then return False;
          else Window:= Work;
          end if;
        end loop Inner;
      end loop;
      return True;
    end;

    function Max_RM is new Max(Test_RM);

    function Test_DDS(CS: Integer) return Boolean is
      TK, U: float;
      TS: constant float:= float(Server_Period);
    begin
      for K in Process_List'range loop
        U:= 0.0;
        for I in Process_List'first..K loop
          U:=U+float(Processes(R)(I).Exec_Time)/float(Processes(R)(I).Period);
        end loop;
        TK:= float(Processes(R)(K).Period);
        if U+(float(CS)/TS)*(1.0+(TS-float(CS))/TK)>1.0 then
          return False;
        end if;
      end loop;
      return True;
    end Test_DDS;

    function Max_DDS is new Max(Test_DDS);

    function Test_XS(CS: Integer) return Boolean is
      Window: Integer:= 0;
      Work: Integer;
    begin
      for K in Process_List'range loop
        Window:= 0;
        Inner: -- check schedulability of Kth process
        loop
          Work:= CS+Ceiling_Div((Window-CS)*CS,Server_Period);
          for I in Process_List'first..K loop
            Work:= Work+Ceiling_Div(Window,Processes(R)(I).Period)
              *Processes(R)(I).Exec_Time;
          end loop;
          if Work<=Window then exit Inner;
          elsif Work>Processes(R)(K).Period then return False;
          else Window:= Work;
          end if;
        end loop Inner;
      end loop;
      return True;
    end;

    function Max_XS is new Max(Test_XS);

  begin
    put("Run #"&Integer'image(R));
    put_line(" periodic utilization "&integer'image(U(R)));
    M:= Max_PLS;
    put(float(M)/float(Server_Period),3,2,0); put("  ");
    put(" EDF-PLS,DSS,DXS: ");
    put_line(Integer'Image(M));
    M:= Max_DDS;
    put(float(M)/float(Server_Period),3,2,0); put("  ");
    put(" EDF-DDS:         ");
    put_line(Integer'Image(M));
    M:= Max_RM;
    put(float(M)/float(Server_Period),3,2,0); put("  ");
    put(" RM-PLS,RMSS:     ");
    put_line(Integer'Image(M));
    M:= Max_DS;
    put(float(M)/float(Server_Period),3,2,0); put("  ");
    put(" RM-DS:           ");
    put_line(Integer'Image(M));
    M:= Max_XS;
    put(float(M)/float(Server_Period),3,2,0); put("  ");
    put(" RM-XS:           ");
    put_line(Integer'Image(M));
  end;

  function MM1 (Rho, EA: Float) return Integer is
    -- Rho = traffic intensity = aperiodic load
    W: constant Float:= (Rho*EA)/(1.0-Rho);
  begin
    if W>Float(Integer(W)) then return Integer(W)+1;
    else return Integer(W);
    end if;
  end MM1;

--         TK:= float(Processes(R)(I).Period);
--         X:= TS+TK;
--         D:= SQRT(X*X-4.0*TK*TS*(1.0-U));
--         CS:= Integer((X-D)/2.0-0.5);
--         if CS<M then M:= CS; end if;

begin
  Calculate_Major_Period(Process_List_40);
  put(Integer'image(Major_Period*50)); new_line;
  Calculate_Major_Period(Process_List_69);
  put(Integer'image(Major_Period*50)); new_line;
  Calculate_Major_Period(Process_List_88);
  put(Integer'image(Major_Period*50)); new_line;
  for R in Runs loop Find_Server_Sizes(R); end loop;
end;



