-- $Id: test_rms.adb,v 1.2 2003/11/18 02:36:14 baker Exp baker $
-- version for Rate-Monotonic Simulation

with Ada.Text_IO; use Ada.Text_IO;
with Ada.Float_Text_IO; use Ada.Float_Text_IO;
with Ada.Numerics.Elementary_Functions;
with Ada.Command_Line;
with System_Type; use System_Type;
with Random_Tools; use Random_Tools;
with BGS; use BGS;
with PLS; use PLS;
with DS; use DS;
with SS; use SS;
with XS; use XS;
with Sizes; use Sizes;

procedure Test_RMS is

use Int_IO;

Prefix : constant String := "rms";

procedure Set_Up (List: Process_List) is
begin
   Start (3, 5, 7, 11, false);
--   Start (Default_I, Default_J, Default_K, Default_L, false);
   Calculate_Major_Period (List);
   End_Of_Test := Major_Period*Repeat;
end Set_Up;

function Plot_File_Name (Run: Runs; IAT: IATS) return String is
begin
  return Prefix & character'val (character'pos ('A')+Run-1) &
         character'val (character'pos ('A')+IAT-1);
end Plot_File_Name;

procedure Put_Plot (F: Float) is
begin
  if F<=0.0 then put (Plot, "99999.999");
  else put (Plot, F, 4, 3, 0);
  end if;
  put (Plot, " ");
end Put_Plot;

procedure Chk (I: IATS; R: Runs; L: Loads; M: Float) is
-- check for average response times that are better than M/M/1
begin
  for S in Sizes.Servers loop
     if Avg (S)(R)(L)<M then
        put (Errs, Plot_File_Name (R, I)); put (Errs, ' ');
        put (Errs, Util (Sizes.BGS)(R)(L), 4, 3, 0); put (Errs, ' ');
        put (Errs, Sizes.Servers'image (S));  put (Errs, " is below M/M/1 by ");
        put (Errs, M-Avg (S)(R)(L), 2, 3, 0); new_line (Errs);
     end if;
  end loop;
end Chk;

procedure Put_Tex (S: Sizes.Servers; R: Runs; L: Loads) is
  F: constant Float := Avg (S)(R)(L);
  use Ada.Numerics.Elementary_Functions;
begin
  if F>0.0 then
     put (Tex, " & "); put (Tex, Integer (F), 5); put (Tex, " & $\PM$ ");
     put (Tex, Integer (Z99*SD (S)(R)(L)/Sqrt (float (Comptd (S)(R)(L)))), 3);
--     put (Tex, Integer (3.0*SD (S)(R)(L)/Sqrt (float (Comptd (S)(R)(L)))), 3);
  else
     put (Tex, " & \overload & \overload ");
  end if;
end Put_Tex;

begin

  begin
     create (Errs, out_file, Prefix & ".errors");
  exception when others=>
     open (Errs, out_file, Prefix & ".errors");
  end;

  declare
    Last: Integer;
    use Ada.Command_Line;
  begin
     if Argument_Count > 1 then
        get (Argument (1), Repeat, Last);
     end if;
  exception when others=>
     Put_Line ("bad repetition count");
     raise;
  end;

  for I in IATS loop
    begin
      create (Tex, out_file,
             Prefix & character'val (character'pos ('A')+I-1)&".tex");
    exception when others=>
      open (Tex, out_file,
           Prefix & character'val (character'pos ('A')+I-1)&".tex");
    end;

    put_line (Tex, "\begin{table}\begin{center}");
    put_line (Tex, "\begin{tabular}{ccr@{}rr@{}rr@{}rr@{}rr@{}r}");
    put_line (Tex, "periodic&aperiodic&");
    put_line (Tex, "\twocol{BGS}&\twocol{PLS}&\twocol{DS}&");
    put_line (Tex, "\twocol{SS}&\twocol{XS}\\[10pt]\hline");
    for R in Runs loop
      put_line (Tex, "\\ ");
      for J in Loads loop
        Set_Up (Processes (R));
        Background_Server (Interarrival_Time_Data (I), Aperiodic_Load (R)(J),
                Processes (R), Avg (Sizes.BGS)(R)(J), SD (Sizes.BGS)(R)(J),
                Max (Sizes.BGS)(R)(J), Reqstd (Sizes.BGS)(R)(J),
                Comptd (Sizes.BGS)(R)(J),
                Missed (Sizes.BGS)(R)(J), Util (Sizes.BGS)(R)(J));
        Set_Up (Processes (R));
        Polling_Server (Interarrival_Time_Data (I), Aperiodic_Load (R)(J),
                Processes (R), RM_Size (R)(Sizes.PLS),
                Avg (Sizes.PLS)(R)(J), SD (Sizes.PLS)(R)(J),
                Max (Sizes.PLS)(R)(J), Reqstd (Sizes.PLS)(R)(J),
                Comptd (Sizes.PLS)(R)(J),
                Missed (Sizes.PLS)(R)(J), Util (Sizes.PLS)(R)(J));
        Set_Up (Processes (R));
        Deferrable_Server (Interarrival_Time_Data (I), Aperiodic_Load (R)(J),
                Processes (R), RM_Size (R)(Sizes.DDS),
                Avg (Sizes.DDS)(R)(J), SD (Sizes.DDS)(R)(J),
                Max (Sizes.DDS)(R)(J), Reqstd (Sizes.DDS)(R)(J),
                Comptd (Sizes.DDS)(R)(J),
                Missed (Sizes.DDS)(R)(J), Util (Sizes.DDS)(R)(J));
        Set_Up (Processes (R));
        Sporadic_Server (Interarrival_Time_Data (I), Aperiodic_Load (R)(J),
                Processes (R), RM_Size (R)(Sizes.DSS),
                Avg (Sizes.DSS)(R)(J), SD (Sizes.DSS)(R)(J),
                Max (Sizes.DSS)(R)(J), Reqstd (Sizes.DSS)(R)(J),
                Comptd (Sizes.DSS)(R)(J),
                Missed (Sizes.DSS)(R)(J), Util (Sizes.DSS)(R)(J));
        Set_Up (Processes (R));
        Exchange_Server (Interarrival_Time_Data (I), Aperiodic_Load (R)(J),
                Processes (R), RM_Size (R)(Sizes.DXS),
                Avg (Sizes.DXS)(R)(J), SD (Sizes.DXS)(R)(J),
                Max (Sizes.DXS)(R)(J), Reqstd (Sizes.DXS)(R)(J),
                Comptd (Sizes.DXS)(R)(J),
                Missed (Sizes.DXS)(R)(J), Util (Sizes.DXS)(R)(J));
      end loop;

      for J in Loads loop
          begin
             put (Tex, float (U (R))/100.0, 1, 2, 0);
             put (Tex, " & ");
             put (Tex, Util (Sizes.BGS)(R)(J), 1, 2, 0);
             for S in Sizes.Servers loop
                Put_Tex (S, R, J);
             end loop;
             put (Tex, "\\");
             new_line (Tex);
          exception
             when others=> put_line (Tex, "exception in output");
          end;
      end loop;
      put_line (Tex, "\\");

    end loop;
    put (Tex, "\end{tabular}\end{center}");
    put (Tex, "\caption{RM response times, mean aperiodic IAT ");
    put (Tex, integer (float (End_Of_Test)/float (Reqstd (Sizes.DXS)(I)(1))), 1);
    put (Tex, ", after ");
    put (Tex, End_Of_Test, 1);
    put_line (Tex, " time units.}");
    put (Tex, "\label{tab:");
    put (Tex, character'val (character'pos ('A')+I-1));
    put_line (Tex, "}\end{table}");
    close (Tex);

    for R in Runs loop
      begin
         create (Plot, out_file, Plot_File_Name (R, I));
      exception when others=>
         open (Plot, out_file, Plot_File_Name (R, I));
      end;
      for J in Loads loop
         declare
           T: constant Float := Avg (Sizes.BGS)(R)(J);
           IAT: constant Float := Interarrival_Time_Data (I);
           AL: constant Float := Util (Sizes.BGS)(R)(J);
           M: constant Float := -- M/M/1, for full server
               AL*IAT/ (1.0-AL);
           -- M/M/1, relative to server capacity =
           -- S: constant Float := float (EDF_Size (R)(Sizes.PLS));
           -- M: constant Float := S*AL*IAT/ (S-AL*float (Server_Period));
         begin
            Chk (I, R, J, M);
            put_Plot (AL);                                  -- 1
            put_Plot (T);                                   -- 2
            put_Plot (Avg (Sizes.PLS)(R)(J));               -- 3
            put_Plot (Avg (Sizes.DDS)(R)(J));               -- 4
            put_Plot (Avg (Sizes.DSS)(R)(J));               -- 5
            put_Plot (Avg (Sizes.DXS)(R)(J));               -- 6
            put_Plot (M);                                   -- 7
            put_Plot (Avg (Sizes.PLS)(R)(J)/T);             -- 8
            put_Plot (Avg (Sizes.DDS)(R)(J)/T);             -- 9
            put_Plot (Avg (Sizes.DSS)(R)(J)/T);             -- 10
            put_Plot (Avg (Sizes.DXS)(R)(J)/T);             -- 11
            put_Plot (M/T);                                 -- 12
         end;
         new_line (Plot);
      end loop;
      close (Plot);
    end loop;

  end loop;

end Test_RMS;
